@@ -1,5 +1,6 @@
 from test.TestBase import WorldTestBase
+from ..constants import game_name
 
 
 class BfBBTestBase(WorldTestBase):
-    game = "Battle for Bikini Bottom"
+    game = game_name
